@@ -39,14 +39,14 @@ except ImportError:
 
 import redis
 
-MED_CLIENT = redis.__version__ >= '2.6.'
-NEW_CLIENT = redis.__version__ >= '3.'
+MED_CLIENT = redis.VERSION >= (2,6,0)
+NEW_CLIENT = redis.VERSION >= (3,0,0)
 STRICT_REDIS = redis.StrictRedis if hasattr(redis, 'StrictRedis') else None
 STRICT_PIPE = redis.StrictPipeline if hasattr(redis, 'StrictPipeline') else None
 STRICT_CONNS = STRICT_REDIS, STRICT_PIPE
 
 def _zadd(conn, key, data):
-    if NEW_CLIENT or isinstance(conn, STRICT_CONNS):
+    if NEW_CLIENT:
         return conn.zadd(key, data)
     return conn.zadd(key, **data)
 
@@ -58,7 +58,7 @@ def _setex(conn, key, value, time):
 if list(map(int, redis.__version__.split('.'))) < [2, 4, 12]:
     raise Exception("Upgrade your Redis client to version 2.4.12 or later")
 
-VERSION = '0.30.3'
+VERSION = '0.31.1'
 
 RPQUEUE_CONFIGS = {}
 
@@ -403,12 +403,13 @@ def _get_work(conn, queues=None, timeout=1):
             delay = REGISTRY[item_id].next(sch)
             if delay is not None:
                 # it can be scheduled again, do it
+                st = sch + delay
                 if delay > 0:
-                    _zadd(pipeline, QUEUE_KEY + queue, {item_id: sch + delay})
+                    _zadd(pipeline, QUEUE_KEY + queue, {item_id: st})
                 else:
                     pipeline.rpush(NOW_KEY + queue, item_id)
-                # re-add the call arguments
-                pipeline.hset(MESSAGES_KEY + queue, item_id, json.dumps(work[:4] + [ts]))
+                # reinsert the call arguments
+                pipeline.hset(MESSAGES_KEY + queue, item_id, json.dumps(work[:4] + [st]))
                 pipeline.execute()
         if t != '0':
             work.append(t)
@@ -1047,7 +1048,7 @@ class _ExecutingTask(object):
     __slots__ = 'task', 'taskid', 'args', 'kwargs', 'exp'
     def __init__(self, task, taskid, exp):
         self.task = task
-        self.taskid = taskid.encode('latin-1') if PY3 else taskid
+        self.taskid = (taskid.encode('latin-1') if PY3 else taskid) if taskid is not None else b''
         self.args = None
         self.kwargs = None
         self.exp = exp
@@ -1057,12 +1058,14 @@ class _ExecutingTask(object):
         self.kwargs = kwargs
         CURRENT_TASK.task = self
         k = RESULT_KEY + self.taskid
-        want_status = self.taskid not in REGISTRY
+        want_status = self.taskid not in REGISTRY and self.taskid
         conn = get_connection()
         if want_status:
             _setex(conn, k, '', 60)
 
-        vt = max(kwargs.get('_vis_timeout', 0), 0)
+        kwargs = dict(kwargs)
+        kwargs.pop('_use_dead', None)
+        vt = max(kwargs.pop('_vis_timeout', 0), 0)
 
         try:
             result = self.task.function(*args, **kwargs)
@@ -1070,7 +1073,11 @@ class _ExecutingTask(object):
             if want_status:
                 conn.delete(k)
             raise
-        del CURRENT_TASK.task
+        try:
+            del CURRENT_TASK.task
+        except AttributeError:
+            # When is thread local not thread local?
+            pass
         pipe = conn.pipeline(False)
         if vt > 0:
             _finish_step_lua(pipe,
